@@ -209,10 +209,17 @@ INSERT INTO %s VALUES (0);''' % (seq_name, seq_name)
     def sql_create_index(self, table, column, unique=False):
         idx = self._index_name(table, column, unique)
         if unique:
-            return 'CREATE UNIQUE INDEX %s ON %s(%s);' % (idx, table, column)
+            return 'ALTER TABLE %s ADD UNIQUE(%s)' % (table, column)
         else:
             return 'CREATE INDEX %s ON %s(%s);' % (idx, table, column)
-    
+
+    def sql_drop_index(self, table, column, unique=False):
+        idx = self._index_name(table, column, unique)
+        if unique:
+            return 'ALTER TABLE %s DROP CONSTRAINT %s' % (table, idx)
+        else:
+            return 'DROP INDEX %s' % idx
+        
     def sql_drop_sequence(self, seq_name):
         return 'DROP TABLE %s;' % seq_name
     
@@ -256,8 +263,7 @@ INSERT INTO %s VALUES (0);''' % (seq_name, seq_name)
             
     def drop_index(self, cursor, table, column, unique=False):
         if self.index_exists(cursor, table, column, unique):
-            idx = self._index_name(table, column, unique)
-            cursor.execute('DROP INDEX %s' % idx)
+            cursor.execute(self.sql_drop_index(table, column, unique))
         
     def index_exists(self, cursor, table, column, unique=False):
         idx = self._index_name(table, column, unique)
@@ -366,7 +372,6 @@ class _PGAdvFuncHelper(_GenericAdvFuncHelper):
             sql += " WITH OWNER=%(owner)s"
         if encoding:
             sql += " ENCODING='%(encoding)s'"
-        print sql % locals()
         cursor.execute(sql % locals())
 
     def create_language(self, cursor, extlang):
@@ -422,6 +427,16 @@ class _SqliteAdvFuncHelper(_GenericAdvFuncHelper):
     ilike_support = False
     union_parentheses_support = False
     
+    def sql_create_index(self, table, column, unique=False):
+        idx = self._index_name(table, column, unique)
+        if unique:
+            return 'CREATE UNIQUE INDEX %s ON %s(%s);' % (idx, table, column)
+        else:
+            return 'CREATE INDEX %s ON %s(%s);' % (idx, table, column)
+
+    def sql_drop_index(self, table, column, unique=False):
+        return 'DROP INDEX %s' % self._index_name(table, column, unique)
+    
     def list_tables(self, cursor):
         """return the list of tables of a database"""
         # filter type='table' else we get indices as well
