@@ -1,25 +1,16 @@
-# This program is free software; you can redistribute it and/or modify it under
-# the terms of the GNU General Public License as published by the Free Software
-# Foundation; either version 2 of the License, or (at your option) any later
-# version.
-#
-# This program is distributed in the hope that it will be useful, but WITHOUT
-# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License along with
-# this program; if not, write to the Free Software Foundation, Inc.,
-# 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
-"""provides helper functions to handle a command line tool providing more than
-one command
+"""Helper functions to support command line tools providing more than
+one command.
+
 e.g called as "tool command [options] args..." where <options> and <args> are
 command'specific
 
-:author:    Logilab
-:copyright: 2003-2008 LOGILAB S.A. (Paris, FRANCE)
-:contact:   http://www.logilab.fr/ -- mailto:python-projects@logilab.org
+:copyright: 2003-2008 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
+:contact: http://www.logilab.fr/ -- mailto:contact@logilab.fr
+:license: General Public License version 2 - http://www.gnu.org/licenses
 """
+__docformat__ = "restructuredtext en"
 
+# XXX : merge with optparser ? 
 import sys
 from os.path import basename
 
@@ -31,14 +22,18 @@ Copyright (c) 2004-2008 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
 http://www.logilab.fr/ -- mailto:contact@logilab.fr'''
 
 
+DEFAULT_DOC = '''\
+Type "%prog <command> --help" for more information about a specific
+command. Available commands are :\n'''
+
 class BadCommandUsage(Exception):
     """Raised when an unknown command is used or when a command is not
-    correctly used
+    correctly used.
     """
 
 
 class Command(Configuration):
-    """base class for command line commands"""
+    """Base class for command line commands."""
     arguments = ''
     name = ''
     # hidden from help ?
@@ -73,7 +68,7 @@ def pop_arg(args_list, expected_size_after=0, msg="Missing argument"):
     except IndexError:
         raise BadCommandUsage(msg)
     if expected_size_after is not None and len(args_list) > expected_size_after:
-        raise BadCommandUsage('Too much arguments')
+        raise BadCommandUsage('too many arguments')
     return value
 
 
@@ -85,16 +80,21 @@ def register_commands(commands):
         _COMMANDS[command_klass.name] = command_klass
 
 
-def main_usage(status=0, __doc__=None, copyright=DEFAULT_COPYRIGHT):
+def main_usage(status=0, __doc__=DEFAULT_DOC, copyright=DEFAULT_COPYRIGHT):
     """display usage for the main program (ie when no command supplied)
     and exit
     """
     commands = _COMMANDS.keys()
     commands.sort()
-    doc = __doc__ % ('<command>', '<command arguments>',
-                     '''\
+    doc = __doc__
+    if doc != DEFAULT_DOC:
+        try:
+            doc = __doc__ % ('<command>', '<command arguments>',
+                             '''\
 Type "%prog <command> --help" for more information about a specific
 command. Available commands are :\n''')
+        except TypeError:
+            print 'could not find the "command", "arguments" and "default" slots'
     doc = doc.replace('%prog', basename(sys.argv[0]))
     print 'usage:', doc
     max_len = max([len(cmd) for cmd in commands]) # list comprehension for py 2.3 support
@@ -124,7 +124,7 @@ def cmd_run(cmdname, *args):
         print command.help()
 
         
-def main_run(args, doc):
+def main_run(args, doc=DEFAULT_DOC):
     """command line tool"""
     try:
         arg = args.pop(0)
