@@ -1,24 +1,9 @@
-# Copyright (c) 2003-2008 LOGILAB S.A. (Paris, FRANCE).
-# http://www.logilab.fr/ -- mailto:contact@logilab.fr
-#
-# This program is free software; you can redistribute it and/or modify it under
-# the terms of the GNU General Public License as published by the Free Software
-# Foundation; either version 2 of the License, or (at your option) any later
-# version.
-#
-# This program is distributed in the hope that it will be useful, but WITHOUT
-# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License along with
-# this program; if not, write to the Free Software Foundation, Inc.,
-# 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 """Command line interface helper classes.
  
- It provides some default commands, a help system, a default readline
- configuration with completion and persistent history
+It provides some default commands, a help system, a default readline
+configuration with completion and persistent history.
 
-Exemple usage:
+Example::
 
     class BookShell(CLIHelper):
 
@@ -39,11 +24,11 @@ Exemple usage:
 
     cl = BookShell()
 
-:author:    Logilab
-:copyright: 2003-2008 LOGILAB S.A. (Paris, FRANCE)
-:contact:   http://www.logilab.fr/ -- mailto:python-projects@logilab.org
+:copyright: 2003-2008 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
+:contact: http://www.logilab.fr/ -- mailto:contact@logilab.fr
+:license: General Public License version 2 - http://www.gnu.org/licenses
 """
-
+__docformat__ = "restructuredtext en"
 
 import __builtin__
 if not hasattr(__builtin__, '_'):
@@ -51,7 +36,7 @@ if not hasattr(__builtin__, '_'):
     
 
 def init_readline(complete_method, histfile=None):
-    """init the readline library if available"""
+    """Init the readline library if available."""
     try:
         import readline
         readline.parse_and_bind("tab: complete")
@@ -70,13 +55,13 @@ def init_readline(complete_method, histfile=None):
 
 
 class Completer :
-    """readline completer"""
+    """Readline completer."""
     
     def __init__(self, commands):
         self.list = commands
         
     def complete(self, text, state):
-        """hook called by readline when <tab> is pressed"""
+        """Hook called by readline when <tab> is pressed."""
         n = len(text)
         matches = []
         for cmd in self.list :
@@ -89,8 +74,8 @@ class Completer :
 
 
 class CLIHelper:
-    """ an abstract command line interface client which recognize commands
-    and provide an help system
+    """An abstract command line interface client which recognize commands
+    and provide an help system.
     """
     
     CMD_MAP = {'help' : _("Others"),
@@ -116,7 +101,7 @@ class CLIHelper:
             if not s_line:
                 continue
             args = s_line.split()
-            if self.commands.has_key(args[0]):
+            if args[0] in self.commands:
                 try:
                     cmd = 'do_%s' % self.commands[args[0]]
                     getattr(self, cmd)(*args[1:])
@@ -133,9 +118,8 @@ class CLIHelper:
                     traceback.print_exc()
 
     def handle_line(self, stripped_line):
-        """method to overload in the concrete class
-        
-        should handle lines wich are not command
+        """Method to overload in the concrete class (should handle
+        lines wich are not commands).
         """
         raise NotImplementedError()
 
@@ -168,9 +152,9 @@ class CLIHelper:
     
     def do_help(self, command=None) :
         """base input of the help system"""
-        if self._command_help.has_key(command):
+        if command in self._command_help:
             self._print_help(*self._command_help[command])
-        elif command is None or not self._topics.has_key(command):
+        elif command is None or command not in self._topics:
             print _("Use help <topic> or help <command>.")
             print _("Available topics are:")
             topics = self._topics.keys()
