@@ -1,27 +1,18 @@
 # pylint: disable-msg=E0601,W0622,W0611
-#
-# Copyright (c) 2004-2006 LOGILAB S.A. (Paris, FRANCE).
-# http://www.logilab.fr/ -- mailto:contact@logilab.fr
-#
-# This program is free software; you can redistribute it and/or modify it under
-# the terms of the GNU General Public License as published by the Free Software
-# Foundation; either version 2 of the License, or (at your option) any later
-# version.
-#
-# This program is distributed in the hope that it will be useful, but WITHOUT
-# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details
-#
-# You should have received a copy of the GNU General Public License along with
-# this program; if not, write to the Free Software Foundation, Inc.,
-# 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
-"""some wrapper around tools introduced into python 2.3, making them available
-in python 2.2
+"""Wrappers around some builtins introduced in python 2.3, 2.4 and
+2.5, making them available in for earlier versions of python.
+
+:copyright: 2000-2008 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
+:contact: http://www.logilab.fr/ -- mailto:contact@logilab.fr
+:license: General Public License version 2 - http://www.gnu.org/licenses
 """
 from __future__ import generators
+__docformat__ = "restructuredtext en"
 
 from warnings import warn
 
+import __builtin__
+
 from logilab.common.deprecation import class_renamed
 
 try:
@@ -188,6 +179,44 @@ except NameError:
         l2.reverse()
         return l2
 
+try: #
+    max = max
+    max(("ab","cde"),key=len)
+except TypeError:
+    def max( *args, **kargs):
+        if len(args) == 0:
+            raise TypeError("max expected at least 1 arguments, got 0")
+        key= kargs.pop("key", None)
+        #default implementation
+        if key is None:
+            return __builtin__.max(*args,**kargs)
+
+        for karg in kargs:
+            raise TypeError("unexpected keyword argument %s for function max") % karg
+
+        if len(args) == 1:
+            items = iter(args[0])
+        else:
+            items = iter(args)
+        
+        try:
+            best_item = items.next()
+            best_value = key(best_item)
+        except StopIteration:
+            raise ValueError("max() arg is an empty sequence")
+        
+        for item in items:
+            value = key(item)
+            if value > best_value:
+                best_item = item
+                best_value = value
+
+        return best_item
+
+
+
+
+
 # Python2.5 builtins
 try:
     any = any
