@@ -1,25 +1,15 @@
-# Copyright (c) 2006-2006 LOGILAB S.A. (Paris, FRANCE).
-# http://www.logilab.fr/ -- mailto:contact@logilab.fr
-#
-# This program is free software; you can redistribute it and/or modify it under
-# the terms of the GNU General Public License as published by the Free Software
-# Foundation; either version 2 of the License, or (at your option) any later
-# version.
-#
-# This program is distributed in the hope that it will be useful, but WITHOUT
-# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License along with
-# this program; if not, write to the Free Software Foundation, Inc.,
-# 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+"""Date manipulation helper functions.
 
-"""date manipulation helper functions"""
+:copyright: 2006-2008 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
+:contact: http://www.logilab.fr/ -- mailto:contact@logilab.fr
+:license: General Public License version 2 - http://www.gnu.org/licenses
+"""
+__docformat__ = "restructuredtext en"
 
 import math
 
 try:
-    from mx.DateTime import RelativeDateTime, strptime
+    from mx.DateTime import RelativeDateTime, strptime, Date
     STEP = 1
 except ImportError:
     from warnings import warn
@@ -65,11 +55,15 @@ else:
 
     def get_national_holidays(begin, end):
         """return french national days off between begin and end"""
+        begin = Date(begin.year, begin.month, begin.day)
+        end = Date(end.year, end.month, end.day)
         holidays = [strptime(datestr, '%Y-%m-%d')
                     for datestr in FRENCH_MOBILE_HOLIDAYS.values()]
         for year in xrange(begin.year, end.year+1):
-            holidays += [strptime(datestr % year, '%Y-%m-%d')
-                         for datestr in FRENCH_FIXED_HOLIDAYS.values()]
+            for datestr in FRENCH_FIXED_HOLIDAYS.values():
+                date = strptime(datestr % year, '%Y-%m-%d')
+                if date not in holidays:
+                    holidays.append(date)
         return [day for day in holidays if begin <= day < end]
 
 
@@ -77,10 +71,12 @@ else:
         """adds date but try to only take days worked into account"""
         weeks, plus = divmod(days, 5)
         end = start+(weeks * 7) + plus
-        if end.day_of_week > 4:
-            end += 2       
+        if end.day_of_week >= 5: # saturday or sunday
+            end += 2
         end += len([x for x in get_national_holidays(start, end+1)
                     if x.day_of_week < 5])
+        if end.day_of_week >= 5: # saturday or sunday
+            end += 2
         return end
 
     def nb_open_days(start, end):
@@ -92,19 +88,11 @@ else:
         elif end.day_of_week == 6:
             plus -= 1
         open_days = weeks * 5 + plus
-        nb_week_holidays = len([x for x in get_national_holidays(start,
-                                                                 end+1)
-                                if x.day_of_week < 5])
+        nb_week_holidays = len([x for x in get_national_holidays(start, end+1)
+                                if x.day_of_week < 5 and x < end])
         return open_days  - nb_week_holidays
 
-    #def nb_open_days(start, end):
-    #    """ brute-force version """
-    #    days = [x for x in date_range(start, end) if x.day_of_week < 5]
-    #    nb_week_holidays = len([x for x in get_national_holidays(start,
-    #                                                             end+1)
-    #                            if x.day_of_week < 5])
-    #    return len(days) - nb_week_holidays
-    
+
 def date_range(begin, end, step=STEP):
     """
     enumerate dates between begin and end dates.
