@@ -12,6 +12,7 @@ __metaclass__ = type
 
 import os.path as osp
 import os
+import tempfile
 
 def escape(value):
     """Make <value> usable in a dot file."""
@@ -53,7 +54,7 @@ class DotBackend:
     def get_source(self):
         """returns self._source"""
         if self._source is None:
-            self.emit("}")
+            self.emit("}\n")
             self._source = '\n'.join(self.lines)
             del self.lines
         return self._source
@@ -69,23 +70,27 @@ class DotBackend:
         :rtype: str
         :return: a path to the generated file
         """
+        name = self.graphname
+        dotfile = dotfile or ('%s.dot' % name)
         if outputfile is not None:
             storedir, basename, target = target_info_from_filename(outputfile)
+            if target != "dot":
+                pdot, dot_sourcepath = tempfile.mkstemp(".dot", name)
+            else:
+                dot_sourcepath = osp.join(storedir, dotfile)
         else:
-            storedir = '/tmp'
-            basename = '%s.png' % (self.graphname)
             target = 'png'
-            outputfile = osp.join(storedir, basename)
-        dotfile = dotfile or ('%s.dot' % self.graphname)
-        dot_sourcepath = osp.join(storedir, dotfile)
-        pdot = file(dot_sourcepath, 'w')
+            pdot, dot_sourcepath = tempfile.mkstemp(".dot", name)
+            ppng, outputfile = tempfile.mkstemp(".png", name)
+        pdot = open(dot_sourcepath,'w')
         if isinstance(self.source, unicode):
             pdot.write(self.source.encode('UTF8'))
         else:
             pdot.write(self.source)
         pdot.close()
         if target != 'dot':
-            os.system('%s -T%s %s -o%s' % (self.renderer, target, dot_sourcepath, outputfile))
+            os.system('%s -T%s %s -o%s' % (self.renderer, target, 
+                        dot_sourcepath, outputfile))
             os.unlink(dot_sourcepath)
         return outputfile
 
@@ -99,8 +104,8 @@ class DotBackend:
         Authorized props: see http://www.graphviz.org/doc/info/attrs.html
         """
         attrs = ['%s="%s"' % (prop, value) for prop, value in props.items()]
-        self.emit('edge [%s];' % ", ".join(attrs))
-        self.emit('%s -> %s' % (normalize_node_id(name1), normalize_node_id(name2)))
+        n_from, n_to = normalize_node_id(name1), normalize_node_id(name2)
+        self.emit('%s -> %s [%s];' % (n_from, n_to, ", ".join(attrs)) )
 
     def emit_node(self, name, **props):
         """Authorized props: see http://www.graphviz.org/doc/info/attrs.html
@@ -170,3 +175,22 @@ def _get_cycles(graph_dict, vertice=None, path=None, result=None):
     except KeyError:
         pass
     path.pop()
+
+def has_path(graph_dict, fromnode, tonode, path=None):
+    """generic function taking a simple graph definition as a dictionary, with
+    node has key associated to a list of nodes directly reachable from it.
+    
+    Return None if no path exists to go from `fromnode` to `tonode`, else the
+    first path found
+    """
+    if path is None:
+        path = []
+    elif fromnode in path:
+        return False
+    path.append(fromnode)
+    for destnode in graph_dict[fromnode]:
+        if destnode == tonode or has_path(graph_dict, destnode, tonode, path):
+            return path[1:]
+    path.pop()
+    return None
+
