@@ -29,7 +29,7 @@ try:
     # python >= 2.3
     from optparse import OptionParser as BaseParser, Option as BaseOption, \
          OptionGroup, OptionValueError, OptionError, Values, HelpFormatter, \
-         NO_DEFAULT
+         NO_DEFAULT, SUPPRESS_HELP 
 except ImportError:
     # python < 2.3
     from optik import OptionParser as BaseParser, Option as BaseOption, \
@@ -153,6 +153,7 @@ class Option(BaseOption):
     """
     TYPES = BaseOption.TYPES + ('regexp', 'csv', 'yn', 'named', 'password',
                                 'multiple_choice', 'file', 'font', 'color')
+    ATTRS = BaseOption.ATTRS + ['hide']
     TYPE_CHECKER = copy(BaseOption.TYPE_CHECKER)
     TYPE_CHECKER['regexp'] = check_regexp
     TYPE_CHECKER['csv'] = check_csv
@@ -166,6 +167,11 @@ class Option(BaseOption):
         TYPES += ('date',)
         TYPE_CHECKER['date'] = check_date
 
+    def __init__(self, *opts, **attrs):
+        BaseOption.__init__(self, *opts, **attrs)
+        if hasattr(self, "hide") and self.hide:
+            self.help = SUPPRESS_HELP 
+
     def _check_choice(self):
         """FIXME: need to override this due to optik misdesign"""
         if self.type in ("choice", "multiple_choice"):
@@ -189,7 +195,7 @@ class Option(BaseOption):
             value = self.convert_value(opt, value)
         except AttributeError: # py < 2.4
             value = self.check_value(opt, value)
-        if self.type == 'named': 
+        if self.type == 'named':
             existant = getattr(values, self.dest)
             if existant:
                 existant.update(value)
@@ -230,7 +236,8 @@ class ManHelpFormatter(HelpFormatter):
         except AttributeError:
             optstring = self.format_option_strings(option)
         if option.help:
-            help = ' '.join([l.strip() for l in option.help.splitlines()])
+            help_text = self.expand_default(option)            
+            help = ' '.join([l.strip() for l in help_text.splitlines()])
         else:
             help = ''
         return '''.IP "%s"
@@ -313,6 +320,7 @@ Please report bugs on the project\'s mailing list:
 def generate_manpage(optparser, pkginfo, section=1, stream=sys.stdout):
     """generate a man page from an optik parser"""
     formatter = ManHelpFormatter()
+    formatter.parser = optparser
     print >> stream, formatter.format_head(optparser, pkginfo, section)
     print >> stream, optparser.format_option_help(formatter)
     print >> stream, formatter.format_tail(pkginfo)
