@@ -1,19 +1,4 @@
-# -*- coding: iso-8859-15 -*-
-# Copyright (c) 2006 LOGILAB S.A. (Paris, FRANCE).
-# http://www.logilab.fr/ -- mailto:contact@logilab.fr
-#
-# This program is free software; you can redistribute it and/or modify it under
-# the terms of the GNU General Public License as published by the Free Software
-# Foundation; either version 2 of the License, or (at your option) any later
-# version.
-#
-# This program is distributed in the hope that it will be useful, but WITHOUT
-# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License along with
-# this program; if not, write to the Free Software Foundation, Inc.,
-# 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+# -*- coding: utf-8 -*-
 """Extend OptionParser with commands.
 
 Example:
@@ -26,9 +11,14 @@ Example:
 >>> return run(options, args[1:])
 
 With mymod.build that defines two functions run and add_options
-"""
 
-# XXX merge with optik_ext ?
+:copyright: 2000-2008 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
+:contact: http://www.logilab.fr/ -- mailto:contact@logilab.fr
+:license: General Public License version 2 - http://www.gnu.org/licenses
+"""
+__docformat__ = "restructuredtext en"
+
+# XXX merge with optik_ext ? merge with clcommands ? 
 
 import sys
 import optparse
