@@ -1,10 +1,102 @@
-"""%prog [OPTIONS] [testfile [testpattern]]
+"""pytest is a tool that eases test running and debugging.
+
+To be able to use pytest, you should either write tests using
+the logilab.common.testlib's framework or the unittest module of the
+Python's standard library.
+
+You can customize pytest's behaviour by defining a ``pytestconf.py`` file
+somewhere in your test directory. In this file, you can add options or
+change the way tests are run.
+
+To add command line options, you must define a ``update_parser`` function in
+your ``pytestconf.py`` file. The function must accept a single parameter
+that will be the OptionParser's instance to customize.
+
+If you wish to customize the tester, you'll have to define a class named
+``CustomPyTester``. This class should extend the default `PyTester` class
+defined in the pytest module. Take a look at the `PyTester` and `DjangoTester`
+classes for more information about what can be done.
+
+For instance, if you wish to add a custom -l option to specify a loglevel, you
+could define the following ``pytestconf.py`` file ::
+
+    import logging
+    from logilab.common.pytest import PyTester
+    
+    def update_parser(parser):
+        parser.add_option('-l', '--loglevel', dest='loglevel', action='store',
+                          choices=('debug', 'info', 'warning', 'error', 'critical'),
+                          default='critical', help="the default log level possible choices are "
+                          "('debug', 'info', 'warning', 'error', 'critical')")
+        return parser
+    
+    
+    class CustomPyTester(PyTester):
+        def __init__(self, cvg, options):
+            super(CustomPyTester, self).__init__(cvg, options)
+            loglevel = options.loglevel.upper()
+            logger = logging.getLogger('erudi')
+            logger.setLevel(logging.getLevelName(loglevel))
+
+
+In your TestCase class you can then get the value of a specific option with
+the ``optval`` method::
+    
+    class MyTestCase(TestCase):
+        def test_foo(self):
+            loglevel = self.optval('loglevel')
+            # ...
+
+
+You can also tag your tag your test for fine filtering
+
+With those tag::
+
+    from logilab.common.testlib import tag, TestCase
+
+    class Exemple(TestCase):
+
+        @tag('rouge', 'carre')
+        def toto(self):
+            pass
+
+        @tag('carre', 'vert')
+        def tata(self):
+            pass
+
+        @tag('rouge')
+        def titi(test):
+            pass
+
+you can filter the function with a simpe python expression
+
+ * ``toto`` and ``titi`` match ``rouge``
+
+ * ``toto``, ``tata`` and ``titi``, match ``rouge or carre``
+
+ * ``tata`` and ``titi`` match``rouge ^ carre``
+
+ * ``titi`` match ``rouge and not carre``
+
+
+
+
+            
+
+:copyright: 2000-2008 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
+:contact: http://www.logilab.fr/ -- mailto:contact@logilab.fr
+:license: General Public License version 2 - http://www.gnu.org/licenses
+"""
+__docformat__ = "restructuredtext en"
+
+PYTEST_DOC = """%prog [OPTIONS] [testfile [testpattern]]
 
 examples:
 
 pytest path/to/mytests.py
 pytest path/to/mytests.py TheseTests
 pytest path/to/mytests.py TheseTests.test_thisone
+pytest path/to/mytests.py -m '(not long and database) or regr'
 
 pytest one (will run both test_thisone and test_thatone)
 pytest path/to/mytests.py -s not (will skip test_notthisone)
@@ -13,10 +105,11 @@ pytest --coverage test_foo.py
   (only if logilab.devtools is available)
 """
 
-import os, sys
+import os, sys, re
 import os.path as osp
 from time import time, clock
 
+
 from logilab.common.fileutils import abspath_listdir
 from logilab.common import testlib
 import doctest
@@ -35,6 +128,7 @@ try:
 except ImportError:
     DJANGO_FOUND = False
 
+CONF_FILE = 'pytestconf.py'
 
 ## coverage hacks, do not read this, do not read this, do not read this
 
@@ -44,7 +138,7 @@ class TraceController(object):
 
     def pause_tracing(cls):
         if not cls.nesting:
-            cls.tracefunc = getattr(sys, '__settrace__', sys.settrace)
+            cls.tracefunc = staticmethod(getattr(sys, '__settrace__', sys.settrace))
             cls.oldtracer = getattr(sys, '__tracer__', None)
             sys.__notrace__ = True
             cls.tracefunc(None)
@@ -95,24 +189,36 @@ else:
 
 
 
+TESTFILE_RE = re.compile("^((unit)?test.*|smoketest)\.py$")
 def this_is_a_testfile(filename):
     """returns True if `filename` seems to be a test file"""
-    filename = osp.basename(filename)
-    return ((filename.startswith('unittest')
-             or filename.startswith('test')
-             or filename.startswith('smoketest')) 
-            and filename.endswith('.py'))
+    return TESTFILE_RE.match(osp.basename(filename))
     
-
+TESTDIR_RE = re.compile("^(unit)?tests?$")
 def this_is_a_testdir(dirpath):
     """returns True if `filename` seems to be a test directory"""
-    return osp.basename(dirpath) in ('test', 'tests', 'unittests')
+    return TESTDIR_RE.match(osp.basename(dirpath))
 
 
-def project_root(projdir=os.getcwd()):
+def load_pytest_conf(path, parser):
+    """loads a ``pytestconf.py`` file and update default parser
+    and / or tester.
+    """
+    namespace = {}
+    execfile(path, namespace)
+    if 'update_parser' in namespace:
+        namespace['update_parser'](parser)
+    return namespace.get('CustomPyTester', PyTester)
+
+
+def project_root(parser, projdir=os.getcwd()):
     """try to find project's root and add it to sys.path"""
     curdir = osp.abspath(projdir)
     previousdir = curdir
+    testercls = PyTester
+    conf_file_path = osp.join(curdir, CONF_FILE)
+    if osp.isfile(conf_file_path):
+        testercls = load_pytest_conf(conf_file_path, parser)
     while this_is_a_testdir(curdir) or \
               osp.isfile(osp.join(curdir, '__init__.py')):
         newdir = osp.normpath(osp.join(curdir, os.pardir))
@@ -120,7 +226,10 @@ def project_root(projdir=os.getcwd()):
             break
         previousdir = curdir
         curdir = newdir
-    return previousdir
+        conf_file_path = osp.join(curdir, CONF_FILE)
+        if osp.isfile(conf_file_path):
+            testercls = load_pytest_conf(conf_file_path, parser)
+    return previousdir, testercls
 
 
 class GlobalTestReport(object):
@@ -173,9 +282,11 @@ class GlobalTestReport(object):
                                                    len(self.errmodules))
             descr = ', '.join(['%s [%s/%s]' % info for info in self.errmodules])
             line3 = '\nfailures: %s' % descr
-        else:
+        elif modulesok:
             line2 = 'All %s modules OK' % modulesok
             line3 = ''
+        else:
+            return ''
         return '%s\n%s%s' % (', '.join(line1), line2, line3)
 
 
@@ -210,11 +321,11 @@ def remove_local_modules_from_sys(testdir):
 class PyTester(object):
     """encaspulates testrun logic"""
     
-    def __init__(self, cvg):
-        self.tested_files = []
+    def __init__(self, cvg, options):
         self.report = GlobalTestReport()
         self.cvg = cvg
-
+        self.options = options
+        self.firstwrite = True
 
     def show_report(self):
         """prints the report and returns appropriate exitcode"""
@@ -228,26 +339,42 @@ class PyTester(object):
         """walks trhough current working directory, finds something
         which can be considered as a testdir and runs every test there
         """
-        for dirname, dirs, files in os.walk(os.getcwd()):
+        here = os.getcwd()
+        for dirname, dirs, _ in os.walk(here):
             for skipped in ('CVS', '.svn', '.hg'):
                 if skipped in dirs:
                     dirs.remove(skipped)
             basename = osp.basename(dirname)
-            if basename in ('test', 'tests'):
+            if this_is_a_testdir(basename):
                 print "going into", dirname
                 # we found a testdir, let's explore it !
                 self.testonedir(dirname, exitfirst)
                 dirs[:] = []
-
+        if self.report.ran == 0:
+            print "no test dir found testing here:", here
+            # if no test was found during the visit, consider
+            # the local directory as a test directory even if
+            # it doesn't have a traditional test directory name
+            self.testonedir(here)
  
     def testonedir(self, testdir, exitfirst=False):
         """finds each testfile in the `testdir` and runs it"""
         for filename in abspath_listdir(testdir):
             if this_is_a_testfile(filename):
+                if self.options.exitfirst and not self.options.restart:
+                    # overwrite restart file
+                    try:
+                        restartfile = open(testlib.FILE_RESTART, "w")
+                        restartfile.close()
+                    except Exception, e:
+                        print >> sys.__stderr__, "Error while overwriting \
+succeeded test file :", osp.join(os.getcwd(),testlib.FILE_RESTART)
+                        raise e
                 # run test and collect information
                 prog = self.testfile(filename, batchmode=True)
                 if exitfirst and (prog is None or not prog.result.wasSuccessful()):
                     break
+                self.firstwrite = True
         # clean local modules
         remove_local_modules_from_sys(testdir)
 
@@ -261,22 +388,39 @@ class PyTester(object):
         dirname = osp.dirname(filename)
         if dirname:
             os.chdir(dirname)
+        # overwrite restart file if it has not been done already
+        if self.options.exitfirst and not self.options.restart and self.firstwrite:
+            try:
+                restartfile = open(testlib.FILE_RESTART, "w")
+                restartfile.close()
+            except Exception, e:
+                print >> sys.__stderr__, "Error while overwriting \
+succeeded test file :", osp.join(os.getcwd(),testlib.FILE_RESTART)
+                raise e
         modname = osp.basename(filename)[:-3]
+        if batchmode:
+            from cStringIO import StringIO
+            outstream = StringIO()
+        else:
+            outstream = sys.stderr
         try:
-            print >>sys.stderr, ('  %s  ' % osp.basename(filename)).center(70, '=')
+            print >> outstream, ('  %s  ' % osp.basename(filename)).center(70, '=')
         except TypeError: # < py 2.4 bw compat
-            print >>sys.stderr, ('  %s  ' % osp.basename(filename)).center(70)
+            print >> outstream, ('  %s  ' % osp.basename(filename)).center(70)
         try:
             try:
                 tstart, cstart = time(), clock()
-                testprog = testlib.unittest_main(modname, batchmode=batchmode, cvg=self.cvg)
+                testprog = testlib.unittest_main(modname, batchmode=batchmode, cvg=self.cvg,
+                                                 options=self.options, outstream=outstream)
                 tend, cend = time(), clock()
                 ttime, ctime = (tend - tstart), (cend - cstart)
+                if testprog.result.testsRun and batchmode:
+                    print >> sys.stderr, outstream.getvalue()
                 self.report.feed(filename, testprog.result, ttime, ctime)
                 return testprog
             except (KeyboardInterrupt, SystemExit):
                 raise
-            except Exception, exc:
+            except Exception:
                 self.report.failed_to_test_module(filename)
                 print 'unhandled exception occured while testing', modname
                 import traceback
@@ -290,10 +434,6 @@ class PyTester(object):
 
 class DjangoTester(PyTester):
 
-    def __init__(self, cvg):
-        super(DjangoTester, self).__init__(cvg)
-
-
     def load_django_settings(self, dirname):
         """try to find project's setting and load it"""
         curdir = osp.abspath(dirname)
@@ -337,7 +477,7 @@ class DjangoTester(PyTester):
         """walks trhough current working directory, finds something
         which can be considered as a testdir and runs every test there
         """
-        for dirname, dirs, files in os.walk(os.getcwd()):
+        for dirname, dirs, _ in os.walk(os.getcwd()):
             for skipped in ('CVS', '.svn', '.hg'):
                 if skipped in dirs:
                     dirs.remove(skipped)
@@ -409,25 +549,23 @@ class DjangoTester(PyTester):
                 os.chdir(here)
 
 
-
-def parseargs():
-    """Parse the command line and return (options processed), (options to pass to
-    unittest_main()), (explicitfile or None).
+def make_parser():
+    """creates the OptionParser instance
     """
     from optparse import OptionParser
-    parser = OptionParser(usage=__doc__)
+    parser = OptionParser(usage=PYTEST_DOC)
 
-    newargs = []
+    parser.newargs = []
     def rebuild_cmdline(option, opt, value, parser):
         """carry the option to unittest_main"""
-        newargs.append(opt)
+        parser.newargs.append(opt)
         
 
     def rebuild_and_store(option, opt, value, parser):
         """carry the option to unittest_main and store
         the value on current parser
         """
-        newargs.append(opt)
+        parser.newargs.append(opt)
         setattr(parser.values, option.dest, True)
 
     # pytest options
@@ -442,9 +580,14 @@ def parseargs():
                       dest="pdb", action="callback",
                       help="Enable test failure inspection (conflicts with --coverage)")
     parser.add_option('-x', '--exitfirst', callback=rebuild_and_store,
-                      dest="exitfirst",
+                      dest="exitfirst", default=False,
                       action="callback", help="Exit on first failure "
                       "(only make sense when pytest run one test file)")
+    parser.add_option('-R', '--restart', callback=rebuild_and_store,
+                      dest="restart", default=False,
+                      action="callback",
+                      help="Restart tests from where it failed (implies exitfirst) "
+                        "(only make sense if tests previously ran with exitfirst only)")
     parser.add_option('-c', '--capture', callback=rebuild_cmdline,
                       action="callback", 
                       help="Captures and prints standard out/err only on errors "
@@ -467,6 +610,8 @@ def parseargs():
                       action="callback", help="Minimal output")
     parser.add_option('-P', '--profile', default=None, dest='profile',
                       help="Profile execution and store data in the given file")
+    parser.add_option('-m', '--match', default=None, dest='tags_pattern',
+                      help="only execute test whose tag macht the current pattern")
 
     try:
         from logilab.devtools.lib.coverage import Coverage
@@ -480,7 +625,13 @@ def parseargs():
         parser.add_option('-J', '--django', dest='django', default=False,
                           action="store_true",
                           help='use pytest for django test cases')
+    return parser
 
+
+def parseargs(parser):
+    """Parse the command line and return (options processed), (options to pass to
+    unittest_main()), (explicitfile or None).
+    """
     # parse the command line
     options, args = parser.parse_args()
     if options.pdb and getattr(options, 'coverage', False):
@@ -495,35 +646,41 @@ def parseargs():
         explicitfile = None
     # someone wants DBC
     testlib.ENABLE_DBC = options.dbc
+    newargs = parser.newargs
     if options.printonly:
         newargs.extend(['--printonly', options.printonly])
     if options.skipped:
         newargs.extend(['--skip', options.skipped])
+    # restart implies exitfirst
+    if options.restart:
+        options.exitfirst = True
     # append additional args to the new sys.argv and let unittest_main
     # do the rest
     newargs += args
-    return options, newargs, explicitfile 
+    return options, explicitfile
+
 
 
 def run():
-    options, newargs, explicitfile = parseargs()
+    parser = make_parser()
+    rootdir, testercls = project_root(parser)
+    options, explicitfile = parseargs(parser)
     # mock a new command line
-    sys.argv[1:] = newargs
+    sys.argv[1:] = parser.newargs
     covermode = getattr(options, 'coverage', None)
     cvg = None
     if not '' in sys.path:
         sys.path.insert(0, '')    
     if covermode:
-        rootdir = project_root()
         # control_import_coverage(rootdir)
         from logilab.devtools.lib.coverage import Coverage
         cvg = Coverage([rootdir])
         cvg.erase()
         cvg.start()
     if DJANGO_FOUND and options.django:
-        tester = DjangoTester(cvg)
+        tester = DjangoTester(cvg, options)
     else:
-        tester = PyTester(cvg)
+        tester = testercls(cvg, options)
     if explicitfile:
         cmd, args = tester.testfile, (explicitfile,)
     elif options.testdir:
@@ -539,7 +696,7 @@ def run():
                 prof.close()
                 print 'profile data saved in', options.profile
             else:
-                 cmd(*args)           
+                cmd(*args)
         except SystemExit:
             raise
         except:
