@@ -5,7 +5,7 @@ __revision__ = '$Id: unittest_testlib.py,v 1.5 2006-02-09 22:37:46 nico Exp $'
 import unittest
 import os
 import sys
-from os.path import join, dirname, isdir, isfile, abspath
+from os.path import join, dirname, isdir, isfile, abspath, exists
 from cStringIO import StringIO
 import tempfile
 import shutil
@@ -17,9 +17,14 @@ try:
 except NameError:
     __file__ = sys.argv[0]
     
-from logilab.common.testlib import TestCase, TestSuite, SkipAwareTextTestRunner
+
+from unittest import TestSuite
+
+from logilab.common.testlib import TestCase, SkipAwareTextTestRunner
 from logilab.common.testlib import mock_object, NonStrictTestLoader, create_files
 from logilab.common.testlib import capture_stdout, unittest_main, InnerTest
+from logilab.common.testlib import with_tempdir, tag
+from logilab.common.testlib import require_version, require_module
 
 class MockTestCase(TestCase):
     def __init__(self):
@@ -65,8 +70,18 @@ class TestlibTC(TestCase):
 
     capture = True
     
+    def mkdir(self,path):
+        if not exists(path):
+            self._dirs.add(path)
+            os.mkdir(path)
+
     def setUp(self):
         self.tc = MockTestCase()
+        self._dirs = set()
+
+    def tearDown(self):
+        while(self._dirs):
+            shutil.rmtree(self._dirs.pop(), ignore_errors=True)
 
     def test_dict_equals(self):
         """tests TestCase.assertDictEquals"""
@@ -116,36 +131,64 @@ class TestlibTC(TestCase):
         self.assertRaises(AssertionError, self.tc.assertXMLStringWellFormed, invalid)
 
 
-    def test_set_equality_for_lists(self):
+    def test_unordered_equality_for_lists(self):
         l1 = [0, 1, 2]
         l2 = [1, 2, 3]
-        self.assertRaises(AssertionError, self.tc.assertSetEqual, l1, l2)
-        self.tc.assertSetEqual(l1, l1)
-        self.tc.assertSetEqual([], [])
+        self.assertRaises(AssertionError, self.tc.assertUnorderedIterableEquals, l1, l2)
+        self.tc.assertUnorderedIterableEquals(l1, l1)
+        self.tc.assertUnorderedIterableEquals([], [])
         l1 = [0, 1, 1]
         l2 = [0, 1]
-        self.assertRaises(AssertionError, self.tc.assertSetEqual, l1, l2)
-        self.tc.assertSetEqual(l1, l1)
+        self.assertRaises(AssertionError, self.tc.assertUnorderedIterableEquals, l1, l2)
+        self.tc.assertUnorderedIterableEquals(l1, l1)
 
 
-    def test_set_equality_for_dicts(self):
+    def test_unordered_equality_for_dicts(self):
         d1 = {'a' : 1, 'b' : 2}
         d2 = {'a' : 1}
-        self.assertRaises(AssertionError, self.tc.assertSetEqual, d1, d2)
-        self.tc.assertSetEqual(d1, d1)
-        self.tc.assertSetEqual({}, {})
+        self.assertRaises(AssertionError, self.tc.assertUnorderedIterableEquals, d1, d2)
+        self.tc.assertUnorderedIterableEquals(d1, d1)
+        self.tc.assertUnorderedIterableEquals({}, {})
 
-    def test_set_equality_for_iterables(self):
-        self.assertRaises(AssertionError, self.tc.assertSetEqual, xrange(5), xrange(6))
-        self.tc.assertSetEqual(xrange(5), range(5))
-        self.tc.assertSetEqual([], ())
+    def test_unordered_equality_for_iterables(self):
+        self.assertRaises(AssertionError, self.tc.assertUnorderedIterableEquals, xrange(5), xrange(6))
+        self.tc.assertUnorderedIterableEquals(xrange(5), range(5))
+        self.tc.assertUnorderedIterableEquals([], ())
 
+    def test_unordered_equality_for_dicts(self):
+        s1 = set('ab')
+        s2 = set('a')
+        self.assertRaises(AssertionError, self.tc.assertSetEquals, s1, s2)
+        self.tc.assertSetEquals(s1, s1)
+        self.tc.assertSetEquals(set(), set())
+    
     def test_file_equality(self):
         foo = join(dirname(__file__), 'data', 'foo.txt')
         spam = join(dirname(__file__), 'data', 'spam.txt')        
         self.assertRaises(AssertionError, self.tc.assertFileEqual, foo, spam)
         self.tc.assertFileEqual(foo, foo)
+    
+    def test_dir_equality(self):
+        ref = join(dirname(__file__), 'data', 'reference_dir')
+        same = join(dirname(__file__), 'data', 'same_dir')
+        subdir_differ = join(dirname(__file__), 'data', 'subdir_differ_dir')
+        file_differ = join(dirname(__file__), 'data', 'file_differ_dir')
+        content_differ = join(dirname(__file__), 'data', 'content_differ_dir')
+        ed1 = join(dirname(__file__), 'data', 'empty_dir_1')
+        ed2 = join(dirname(__file__), 'data', 'empty_dir_2')
 
+        for path in (ed1, ed2, join(subdir_differ,'unexpected')):
+            self.mkdir(path)
+
+        self.assertDirEquals(ed1, ed2)
+        self.assertDirEquals(ref, ref)
+        self.assertDirEquals( ref, same)
+        self.assertRaises(AssertionError, self.assertDirEquals, ed1, ref)
+        self.assertRaises(AssertionError, self.assertDirEquals, ref, ed2)
+        self.assertRaises(AssertionError, self.assertDirEquals, subdir_differ, ref)
+        self.assertRaises(AssertionError, self.assertDirEquals, file_differ, ref)
+        self.assertRaises(AssertionError, self.assertDirEquals, ref, content_differ)
+        
     def test_stream_equality(self):
         foo = join(dirname(__file__), 'data', 'foo.txt')
         spam = join(dirname(__file__), 'data', 'spam.txt')        
@@ -156,6 +199,11 @@ class TestlibTC(TestCase):
         self.assertRaises(AssertionError, self.tc.assertStreamEqual, stream1, stream2)
         
     def test_text_equality(self):
+        self.assertRaises(AssertionError, self.tc.assertTextEqual, "toto", 12)
+        self.assertRaises(AssertionError, self.tc.assertTextEqual, "toto", None)
+        self.assertRaises(AssertionError, self.tc.assertTextEqual, 3.12, u"toto")
+        self.assertRaises(AssertionError, self.tc.assertTextEqual, None, u"toto")
+        
         foo = join(dirname(__file__), 'data', 'foo.txt')
         spam = join(dirname(__file__), 'data', 'spam.txt')        
         text1 = file(foo).read()
@@ -203,6 +251,34 @@ class TestlibTC(TestCase):
         tc2 = MyTC('test_1')
         self.assertEquals(tc2.datadir, expected_datadir)
         
+    def test_is(self):
+        obj_1 = []
+        obj_2 = []
+        self.assertIs(obj_1,obj_1)
+        self.assertRaises(AssertionError, self.assertIs, obj_1, obj_2)
+    
+    def test_isnot(self):
+        obj_1 = []
+        obj_2 = []
+        self.assertIsNot(obj_1,obj_2)
+        self.assertRaises(AssertionError, self.assertIsNot, obj_1, obj_1)
+
+    def test_none(self):
+        self.assertNone(None)
+        self.assertRaises(AssertionError, self.assertNone, object())
+
+    def test_none(self):
+        self.assertNotNone(object())
+        self.assertRaises(AssertionError, self.assertNotNone, None)
+
+    def test_in(self):
+        self.assertIn("a", "dsqgaqg")
+        obj, seq = 'a', ('toto', "azf", "coin")
+        self.assertRaises(AssertionError, self.assertIn, obj, seq)
+
+    def test_not_in(self):
+        self.assertNotIn('a', ('toto', "azf", "coin"))
+        self.assertRaises(AssertionError, self.assertNotIn, 'a', "dsqgaqg")
 
 
 class GenerativeTestsTC(TestCase):
@@ -524,6 +600,186 @@ class OutErrCaptureTC(TestCase):
         bootstrap_print("hello")
         self.assertEquals(output.restore(), "hello")
         
+    def test_exotic_unicode_string(self):
+        class FooTC(TestCase):
+            def test_xxx(self):
+                raise Exception(u'\xe9')
+        test = FooTC('test_xxx')
+        result = self.runner.run(test)
+
+class DecoratorTC(TestCase):
+    
+    @with_tempdir
+    def test_tmp_dir_normal(self):
+
+
+        tempdir = tempfile.gettempdir()
+        # assert temp directory is empty
+        self.assertListEquals(list(os.walk(tempdir)),
+            [(tempdir,[],[])])
+
+        witness = []
+
+        @with_tempdir
+        def createfile(list):
+            tempfile.mkstemp()
+            tempfile.mkstemp()
+            dir = tempfile.mkdtemp()
+            tempfile.mkstemp(dir=dir)
+            tempfile.mkdtemp()
+            list.append(True)
+
+        self.assertFalse(witness)
+        createfile(witness)
+        self.assertTrue(witness)
+        
+        self.assertEquals(tempfile.gettempdir(), tempdir)
+        
+        # assert temp directory is empty
+        self.assertListEquals(list(os.walk(tempdir)),
+            [(tempdir,[],[])])
+
+    @with_tempdir
+    def test_tmp_dir_normal(self):
+
+        tempdir = tempfile.gettempdir()
+        # assert temp directory is empty
+        self.assertListEquals(list(os.walk(tempfile.tempdir)),
+            [(tempfile.tempdir,[],[])])
+
+
+        class WitnessException(Exception):
+            pass
+
+        @with_tempdir
+        def createfile():
+            tempfile.mkstemp()
+            tempfile.mkstemp()
+            dir = tempfile.mkdtemp()
+            tempfile.mkstemp(dir=dir)
+            tempfile.mkdtemp()
+            raise WitnessException()
+
+        self.assertRaises(WitnessException, createfile)
+        
+
+        # assert tempdir didn't change
+        self.assertEquals(tempfile.gettempdir(), tempdir)
+
+        # assert temp directory is empty
+        self.assertListEquals(list(os.walk(tempdir)),
+            [(tempdir,[],[])])
+
+
+    def setUp(self):
+        self.pyversion = sys.version_info
+
+    def tearDown(self):
+        sys.version_info = self.pyversion
+
+    def test_require_version_good(self):
+        """ should return the same function
+        """
+        def func() :
+            pass
+        sys.version_info = (2, 5, 5, 'final', 4)
+        current = sys.version_info[:3]
+        compare = ('2.4', '2.5', '2.5.4', '2.5.5')
+        for version in compare: 
+            decorator = require_version(version)
+            self.assertEquals(func, decorator(func), '%s =< %s : function \
+                return by the decorator should be the same.' % (version, 
+                '.'.join([str(element) for element in current])))
+
+    def test_require_version_bad(self):
+        """ should return a different function : skipping test
+        """
+        def func() :
+            pass
+        sys.version_info = (2, 5, 5, 'final', 4)
+        current = sys.version_info[:3]
+        compare = ('2.5.6', '2.6', '2.6.5')
+        for version in compare: 
+            decorator = require_version(version)
+            self.assertNotEquals(func, decorator(func), '%s >= %s : function \
+                 return by the decorator should NOT be the same.' 
+                 % ('.'.join([str(element) for element in current]), version))
+
+    def test_require_version_exception(self):
+        """ should throw a ValueError exception
+        """
+        def func() :
+            pass
+        compare = ('2.5.a', '2.a', 'azerty')
+        for version in compare: 
+            decorator = require_version(version)
+            self.assertRaises(ValueError, decorator, func)
+
+    def test_require_module_good(self):
+        """ should return the same function
+        """
+        def func() :
+            pass
+        module = 'sys'
+        decorator = require_module(module)
+        self.assertEquals(func, decorator(func), 'module %s exists : function \
+            return by the decorator should be the same.' % module)
+
+    def test_require_module_bad(self):
+        """ should return a different function : skipping test
+        """
+        def func() :
+            pass
+        modules = ('bla', 'blo', 'bli')
+        for module in modules:
+            try:
+                __import__(module)
+                pass
+            except ImportError:
+                decorator = require_module(module)
+                self.assertNotEquals(func, decorator(func), 'module %s does \
+                    not exist : function return by the decorator should \
+                    NOT be the same.' % module)
+                return
+        print 'all modules in %s exist. Could not test %s' % (', '.join(modules), 
+            sys._getframe().f_code.co_name)
+
+class TagTC(TestCase):
+
+    def setUp(self):
+        @tag('testing', 'bob')
+        def bob(a, b, c):
+            return (a + b) * c
+
+        self.func = bob
+
+    def test_tag_decorator(self):
+        bob = self.func
+        
+        self.assertEquals(bob(2, 3, 7), 35)
+        self.assertTrue(hasattr(bob, 'tags'))
+        self.assertSetEquals(bob.tags, set(['testing','bob']))
+
+
+    def test_tags_class(self):
+        tags = self.func.tags
+
+        self.assertTrue(tags['testing'])
+        self.assertFalse(tags['Not inside'])
+
+    def test_tags_match(self):
+        tags = self.func.tags
+
+        self.assertTrue(tags.match('testing'))
+        self.assertFalse(tags.match('other'))
+
+        self.assertFalse(tags.match('testing and coin'))
+        self.assertTrue(tags.match('testing or other'))
+
+        self.assertTrue(tags.match('not other'))
+
+        self.assertTrue(tags.match('not other or (testing and bibi)'))
+        self.assertTrue(tags.match('other or (testing and bob)'))
 
 if __name__ == '__main__':
     unittest_main()
