@@ -1,21 +1,10 @@
-# Copyright (c) 2003-2006 LOGILAB S.A. (Paris, FRANCE).
-# http://www.logilab.fr/ -- mailto:contact@logilab.fr
-#
-# This program is free software; you can redistribute it and/or modify it under
-# the terms of the GNU General Public License as published by the Free Software
-# Foundation; either version 2 of the License, or (at your option) any later
-# version.
-#
-# This program is distributed in the hope that it will be useful, but WITHOUT
-# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License along with
-# this program; if not, write to the Free Software Foundation, Inc.,
-# 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
-"""
- base class to represent tree structure
+"""Base class to represent a tree structure.
+
+:copyright: 2000-2008 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
+:contact: http://www.logilab.fr/ -- mailto:contact@logilab.fr
+:license: General Public License version 2 - http://www.gnu.org/licenses
 """
+__docformat__ = "restructuredtext en"
 
 import sys
 
@@ -43,6 +32,9 @@ class Node(object):
         self.parent = None
         self.children = []
 
+    def __iter__(self):
+        return iter(self.children)
+
     def __str__(self, indent=0):
         s = ['%s%s %s' % (' '*indent, self.__class__.__name__, self.id)]
         indent += 2
@@ -53,7 +45,6 @@ class Node(object):
                 s.append(child.__str__())
         return '\n'.join(s)
 
-
     def is_leaf(self):
         return not self.children
     
