@@ -1,6 +1,13 @@
-"""unicode email support"""
+"""Unicode email support (extends email from stdlib).
+
+:copyright: 2000-2008 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
+:contact: http://www.logilab.fr/ -- mailto:contact@logilab.fr
+:license: General Public License version 2 - http://www.gnu.org/licenses
+"""
+__docformat__ = "restructuredtext en"
 
 import email
+from encodings import search_function
 from email.Utils import parseaddr, parsedate
 from email.Header import decode_header
 
@@ -31,7 +38,8 @@ def message_from_string(string):
         return ''
     
 class UMessage:
-    """Encapsulates an email.Message instance and returns only unicode objects"""
+    """Encapsulates an email.Message instance and returns only unicode objects.
+    """
 
     def __init__(self, message):
         self.message = message
@@ -58,9 +66,9 @@ class UMessage:
                 return payload
 
             charset = message.get_content_charset() or 'iso-8859-1'
-            if charset == 'unknown-8bit':
+            if search_function(charset) is None:
                 charset = 'iso-8859-1'
-            return unicode(payload or '', charset)
+            return unicode(payload or '', charset, "replace")
         else:
             payload = UMessage(message.get_payload(index, decode))
         return payload
@@ -109,17 +117,22 @@ class UMessage:
             persons.append((name, mail))
         return persons
     
-    def date(self):
+    def date(self, alternative_source=False, return_str=False):
         """return a mx.DateTime object for the email's date or None if no date is
         set or if it can't be parsed
         """
         value = self.get('date')
-        if value:
+        if value is None and alternative_source:
+            unix_from = self.message.get_unixfrom()
+            if unix_from is not None:
+                try:
+                    value = unix_from.split(" ", 2)[2]
+                except IndexError:
+                    pass
+        if value is not None:
             datetuple = parsedate(value)
             if datetuple:
                 return DateTime(*datetuple[:6])
-        return None
-
-    
-
-    
+            elif not return_str:
+                return None
+        return value
